from distutils.core import setup

setup(
    name='marsha',
    version='0.2.0',
    packages=['marsha', 'marsha.formats'],
    license='MIT',
    url='https://gitlab.com/deckar01/marsha',
    author='Jared Deckard',
    author_email='jared@shademaps.com',
    long_description=open('README.md').read(),
)
